#version 120
// Output generated from file: Examples/Knighty Collection/MandalayBox.frag
// Created: Sat May 9 23:30:11 2015
#info MandalayBox Distance Estimator.
#define providesInit
#define KN_VOLUMETRIC
//#define USE_IQ_CLOUDS
#define WANG_HASH
#include "MathUtils.frag"
uniform float time;
#include "DE-Kn2.frag"
#group MandalayBox

// Number of iterations.
uniform int Iterations;  slider[0,10,20]

// Scale parameter. A perfect Menger is 3.0
uniform float Scale; slider[-6.0,2,6.00]
uniform bool DoBoxFold; checkbox[false]
uniform vec3 RotVector; slider[(0,0,0),(1,0,0),(1,1,1)]

// Scale parameter. A perfect Menger is 3.0
uniform float RotAngle; slider[0.00,00,360]

// coordinate to invert to infinity
uniform vec3 InvertC; slider[(-5,-5,-5),(0,0,0),(5,5,5)]

uniform vec3 fo; slider[(0.,0,0),(0.5,0.5,0.5),(2.,2.,2.)]
uniform vec3 g; slider[(0.,0,0),(0.1,0.1,0.1),(2.,2.,2.)]
uniform bool Serial; checkbox[false]
//Mandelbox's radial fold
uniform float MinRad2;  slider[0,0.25,2.0]
// Julia
uniform bool Julia; checkbox[false]
uniform vec3 JuliaC; slider[(-4,-4,-4),(1,1,1),(4,4,4)]

mat3 rot;
float sr;
void init() {
	rot = rotationMatrix3(normalize(RotVector), RotAngle);
	sr = 30.;
}

//DarkBeam's "fold"... reinterpreted... it's more than a fold, much more! Just awesome!
float DBFold(vec3 p, float fo, float g){
	if(p.z>p.y) p.yz=p.zy;//Diagonal fold
	//Tis is slightly different from the original fold in order to make it continuous in this context
	float vx=p.x-2.*fo;
	float vy=p.y-4.*fo;
 	float v=max(abs(vx+fo)-fo,vy);
	float v1=max(vx-g,p.y);
	v=min(v,v1);
	return min(v,p.x);
}
//the coordinates are pushed/pulled in parallel
vec3 DBFoldParallel(vec3 p, vec3 fo, vec3 g){
	vec3 p1=p;
	p.x=DBFold(p1,fo.x,g.x);
	p.y=DBFold(p1.yzx,fo.y,g.y);
	p.z=DBFold(p1.zxy,fo.z,g.z);
	return p;
}
//serial version
vec3 DBFoldSerial(vec3 p, vec3 fo, vec3 g){
	p.x=DBFold(p,fo.x,g.x);
	p.y=DBFold(p.yzx,fo.y,g.y);
	p.z=DBFold(p.zxy,fo.z,g.z);
	return p;
}

// Roqen's domain mashup performs the active c = T(s)
vec3 domainMap(vec3 c)
{
  float s = dot(c,c);
  return c/s * InvertC;
}

float Coloring(vec3 c)
{
  float s = dot(c,c);
  c = c/s - InvertC;
  return dot(c,c);
}

float DE(vec3 p)
{
	vec4 JC=Julia? vec4(JuliaC,0.) : vec4(p,1.);

     p = InvertC != vec3(0,0,0) ? domainMap(-p) : p;

	float r2=dot(p,p);
	float dd = 1.;
	for(int i = 0; i<Iterations && r2<10000.; i++){

		if(DoBoxFold){
             p = p - clamp(p.xyz, -1.0, 1.0) * 2.0;  // mandelbox's box fold
          }
		//Apply pull transformation
		vec3 signs=sign(p);//Save 	the original signs
		p=abs(p);
		if(Serial) p=DBFoldSerial(p,fo,g);
		else p=DBFoldParallel(p,fo,g);
		p*=signs;//resore signs: this way the mandelbrot set won't extend in negative directions

		//Sphere fold
		r2=dot(p,p);
		float  t = clamp(1./r2, 1., 1./MinRad2);
		p*=t; dd*=t;

//     p = InvertC ? p+domainMap(-p) :p;

		//Scale and shift
		p=p*Scale+JC.xyz; dd=dd*Scale+JC.w;
		p=rot*p;
		//For coloring and bailout
		r2=dot(p,p);
		orbitTrap = min(orbitTrap, abs(vec4(p.x,p.y,p.z,r2)));
	}
	dd=abs(dd);
#if 1
	return (sqrt(r2)-sr)/dd;//bounding volume is a sphere
#else
	p=abs(p); return (max(p.x,max(p.y,p.z))-sr)/dd;//bounding volume is a cube
#endif
}











#preset Default
FOV = 0.6
Eye = -0.7367891,0.8678789,-0.0003532
Target = 0.1829017,0.4653223,-0.0038993
Up = 0.3019272,0.8985473,0
Gamma = 1
ToneMapping = 1
Exposure = 1
Brightness = 1
Contrast = 1
Saturation = 1
GaussianWeight = 1
AntiAliasScale = 2
Detail = -3.3
FudgeFactor = 0.2
Dither = 1
NormalBackStep = 0 NotLocked
CamLight = 1,1,1,1
BaseColor = 1,1,1
OrbitStrength = 0.1973684
X = 0.5,0.6,0.6,0.6
Y = 1,0.6,0,0.5
Z = 0.8,0.78,1,0.5
R = 0.4,0.7,1,-0.4
BackgroundColor = 0.278431,0.278431,0.419608
GradientBackground = 2
CycleColors = true
Cycles = 4.1
EnableFloor = false NotLocked
FloorNormal = 0,1,1
FloorColor = 1,1,1
FocalPlane = 1
Aperture = 0
InFocusAWidth = 0
ApertureNbrSides = 5
ApertureRot = 0
ApStarShaped = false
BloomIntensity = 0
DetailAO = -2.75
coneApertureAO = 0.5
maxIterAO = 20
FudgeAO = 1
AO_ambient = 1
AO_camlight = 1.5
AO_pointlight = 1.5
AoCorrect = 1
Specular = 0.1801802
SpecularExp = 250
AmbiantLight = 1,1,1,2
Reflection = 0.5098039,0.5098039,0.5098039
ReflectionsNumber = 1
SpotGlow = false
LightSize = 0
LightFallOff = 0
LightGlowRad = 0
LightGlowExp = 0.531915
HardShadow = 1
ShadowSoft = 20
HF_Fallof = 5
HF_Const = 0.122449
HF_Intensity = 0.402439
HF_Dir = 0,1,0
HF_Offset = -1
HF_Color = 0.145098,0,0,1
HF_Scatter = 2.170732
HF_Anisotropy = 0,0,0
HF_CastShadow = true
CloudScale = 0.1 NotLocked
CloudTops = 2 NotLocked
CloudBase = 0.3797468 NotLocked
CloudDensity = 0.3815789 NotLocked
CloudRoughness = 0.8846154 NotLocked
CloudContrast = 2.580645 NotLocked
CloudColor = 0.65,0.68,0.7 NotLocked
SunLightColor = 0.7,0.5,0.3 NotLocked
AutoFocus = false
EquiRectangular = false
Bloom = false
BloomPow = 2
BloomTaps = 4
LensFlare = false
FlareIntensity = 0.32
FlareSamples = 8
FlareDispersal = 0.25
FlareHaloWidth = 0.5
FlareDistortion = 1
DepthToAlpha = false
RefineSteps = 4
MaxRaySteps = 1100
MaxDistance = 10
Glow = 1,1,1,0
GlowMax = 233
SpotLight = 1,1,1,10
LightPos = 8.16092,8.390805,7.931034
FloorHeight = 0
HF_FogIter = 6
CloudFlatness = 0.9
WindDir = 0,0,1
WindSpeed = 1
Iterations = 14
Scale = -3
DoBoxFold = false
RotVector = 1,0,0
RotAngle = 0
InvertC = 1,-1,2
fo = 0.8951049,0.8111888,0.7412587
g = 0.1904762,0.2993197,0.707483
Serial = false
MinRad2 = 0.25
Julia = false
JuliaC = 1,1,1
#endpreset

