#donotrun

// http://www.fractalforums.com/index.php?topic=22721.msg88910#msg88910
// required for Wang hash
#extension GL_ARB_shader_bit_encoding : enable
#extension GL_EXT_gpu_shader4 : enable
#extension GL_ARB_gpu_shader5 : enable
#ifdef WANG_HASH
uint wang_hash(uint seed)
{
        seed = (seed ^ 61u) ^ (seed >> 16u);
        seed *= 9u;
        seed = seed ^ (seed >> 4u);
        seed *= 0x27d4eb2du;
        seed = seed ^ (seed >> 15u);
        return seed ;
}


// Wrapper for getting from float to ints. This certainly looses precision. I imagine we could do better here.
float wang_hash_fp(float v)
{
        uint ix = floatBitsToUint(v);
        return float(wang_hash(ix)) / 4294967295.0;
}

vec2 wang_hash_fp(vec2 v)
{
        uint ix = floatBitsToUint(v.x);
        uint iy = floatBitsToUint(v.y);
        return vec2(wang_hash(ix),wang_hash(iy)) / 4294967295.0;
}

vec3 wang_hash_fp(vec3 v)
{
        uint ix = floatBitsToUint(v.x);
        uint iy = floatBitsToUint(v.y);
        uint iz = floatBitsToUint(v.z);
        return vec3(wang_hash(ix),wang_hash(iy),wang_hash(iz)) / 4294967295.0;
}
#endif

// http://marc-b-reynolds.github.io/math/2016/03/29/weyl_hash.html
#define W0 0.5545497
#define W1 0.308517

float hash(in vec2 c)
{
  float x = c.x*fract(c.x * W0);
  float y = c.y*fract(c.y * W1);
  return fract(x*y);
}

// float2 sincos_noise;
// sincos(dot(tex.xy, float2(12.9898,78.233)),sincos_noise.x,sincos_noise.y);
// sincos_noise = frac(sincos_noise * 43758.5453 + tex.yx);

/*
// http://www.fractalforums.com/fragmentarium/better-living-through-random-numbers/msg91843
uint rhash(in uvec2 c) { c = uvec2(0x3504f333, 0xf1bbcdcb)*c; return 741103597*(c.x^c.y); }
// returns [0,1]
float u32tofpi(in uint v) { return (1.0/4294967296.0)*float(v); }
// returns [0,1)
float u32tofp(in uint v) { return (1.0/16777216.0)*float(v>>8); }
// base LCG
uint lcg(in uint v) { return 741103597*v + 0x1234567; }
// return the currect value as a float and then updates
float rng(inout uint v} { float r = u32tofp(v); v = lcg(v); return r; }

void doSomething(vecN coord)
{
    uint seed = rhash(coord);  // use hash to get first integer from coordinate

    float n0    = rng(seed);    // transform 'seed' to first float and update state.
    float n1    = rng(seed);    // second random number...

   // however many more is needed.  This helper function way I've structured
   // this means we will always be doing one more integer random number
   // generating step than needed.  Since I've typed it that way I'll leave as is.
}
*/

// Standard matrices

// Return rotation matrix for rotating around vector v by angle
mat3  rotationMatrix3(vec3 v, float angle)
{
	float c = cos(radians(angle));
	float s = sin(radians(angle));

	return mat3(c + (1.0 - c) * v.x * v.x, (1.0 - c) * v.x * v.y - s * v.z, (1.0 - c) * v.x * v.z + s * v.y,
		(1.0 - c) * v.x * v.y + s * v.z, c + (1.0 - c) * v.y * v.y, (1.0 - c) * v.y * v.z - s * v.x,
		(1.0 - c) * v.x * v.z - s * v.y, (1.0 - c) * v.y * v.z + s * v.x, c + (1.0 - c) * v.z * v.z
		);
}

mat3 rotationMatrixXYZ(vec3 v) {
	return rotationMatrix3(vec3(1.0,0.0,0.0), v.x)*
	rotationMatrix3(vec3(0.0,1.0,0.0), v.y)*
	rotationMatrix3(vec3(0.0,0.0,1.0), v.z);
}

// Return rotation matrix for rotating around vector v by angle
mat4  rotationMatrix(vec3 v, float angle)
{
	float c = cos(radians(angle));
	float s = sin(radians(angle));

	return mat4(c + (1.0 - c) * v.x * v.x, (1.0 - c) * v.x * v.y - s * v.z, (1.0 - c) * v.x * v.z + s * v.y, 0.0,
		(1.0 - c) * v.x * v.y + s * v.z, c + (1.0 - c) * v.y * v.y, (1.0 - c) * v.y * v.z - s * v.x, 0.0,
		(1.0 - c) * v.x * v.z - s * v.y, (1.0 - c) * v.y * v.z + s * v.x, c + (1.0 - c) * v.z * v.z, 0.0,
		0.0, 0.0, 0.0, 1.0);
}

mat4 translate(vec3 v) {
	return mat4(1.0,0.0,0.0,0.0,
		0.0,1.0,0.0,0.0,
		0.0,0.0,1.0,0.0,
		v.x,v.y,v.z,1.0);
}

mat4 scale4(float s) {
	return mat4(s,0.0,0.0,0.0,
		0.0,s,0.0,0.0,
		0.0,0.0,s,0.0,
		0.0,0.0,0.0,1.0);
}


